//
//  Partition Logic
//  Copyright (C) 1998-2013 J. Andrew McLaughlin
// 
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2 of the License, or (at your option)
//  any later version.
// 
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
//  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
//  partlogic.c
//

#include <libintl.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/api.h>


int main(void) __attribute__((noreturn));

#define _(string) gettext(string)
#define WELCOMESTRING _("Welcome to Partition Logic")
#define COPYSTRING _("Copyright (C) 1998-2013 J. Andrew McLaughlin")
#define PLUSSTRING _("PLUS+")
#define GPLSHORTSTRING \
_("  Contains software distributed under the terms of the GNU General  \n"    \
  "  Public license - partitionlogic.org.uk")
#define GPLSTRING \
_("  This program is free software; you can redistribute it and/or modify  \n"\
  "  it under the terms of the GNU General Public License as published by\n"  \
  "  the Free Software Foundation; either version 2 of the License, or\n"     \
  "  (at your option) any later version.\n\n"				      \
  "  This program is distributed in the hope that it will be useful, but\n"   \
  "  WITHOUT ANY WARRANTY; without even the implied warranty of\n"	      \
  "  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See\n"	      \
  "  the file /system/COPYING.txt for more details.")

// GUI stuff
static int graphics = 0;
static int bootGraphicsMode = 0;
static objectKey window = NULL;
static objectKey graphicsCheckbox = NULL;
static objectKey okButton = NULL;
static objectKey cancelButton = NULL;

#ifdef PLUS
extern int licenseCheckKeyValid(void);
#endif


static void setGraphics(int yes)
{
  // This allows the user to toggle text/graphics mode for the next reboot.

  file theFile;
  static char *noGraphName = "/nograph";

  if (yes)
    {
      // Try to delete any 'no graphics' file
      if (fileFind(noGraphName, &theFile) >= 0)
	fileDelete(noGraphName);
    }

  else
    {
      // Try to create a 'no graphics' file
      bzero(&theFile, sizeof(file));
      if (fileFind(noGraphName, &theFile) < 0)
	{
	  fileOpen(noGraphName, (OPENMODE_WRITE | OPENMODE_CREATE), &theFile);
	  fileClose(&theFile);
	}
    }
}


static void eventHandler(objectKey key, windowEvent *event)
{
  // Check for the graphics checkbox
  if ((key == graphicsCheckbox) && (event->type & EVENT_SELECTION))
    {
      windowComponentGetSelected(graphicsCheckbox, &bootGraphicsMode);
      setGraphics(bootGraphicsMode);
    }

  // Check for the 'Ok' button
  if ((key == okButton) && (event->type == EVENT_MOUSE_LEFTUP))
    {
      #ifdef PLUS
      if (licenseCheckKeyValid() < 0)
	{
	  windowNewErrorDialog(window, _("Licence key"),
			       _("The license key you entered is not valid"));
	  return;
	}
      #endif
      windowGuiStop();
    }

  // Check for the 'Cancel' button
  if ((key == cancelButton) && (event->type == EVENT_MOUSE_LEFTUP))
    shutdown(1, 0); 
}


static void constructWindow(const char *windowTitle)
{
  // If we are in graphics mode, make a window rather than operating on the
  // command line.

  image splashImage;
  componentParameters params;
  objectKey font = NULL;

  #ifdef PLUS
  extern objectKey licensePanel(objectKey, componentParameters *);
  #endif

  // Create a new window
  window = windowNew(multitaskerGetCurrentProcessId(), windowTitle);
  if (window == NULL)
    return;

  bzero(&params, sizeof(componentParameters));
  params.gridWidth = 2;
  params.gridHeight = 1;
  params.padTop = 5;
  params.padLeft = 5;
  params.padRight = 5;
  params.orientationX = orient_left;
  params.orientationY = orient_middle;
  windowNewTextLabel(window, COPYSTRING, &params);

  params.gridY += 1;
  params.padBottom = 5;
  params.orientationX = orient_center;
  params.flags |= WINDOW_COMPFLAG_HASBORDER;
  bzero(&splashImage, sizeof(image));
  imageLoad("/system/partlogic.jpg", 0, 0, &splashImage);
  if (splashImage.data != NULL)
    // Create an image component from it, and add it to the window
    windowNewImage(window, &splashImage, draw_translucent, &params);

  params.gridY += 1;
  params.padBottom = 0;
  params.orientationX = orient_left;
  fontLoad("arial-bold-10.bmp", "arial-bold-10", &font, 0);
  params.flags &= ~WINDOW_COMPFLAG_HASBORDER;
  params.font = font;
#ifdef PLUS
  licensePanel(window, &params);

  params.gridY += 1;
  windowNewTextLabel(window, GPLSHORTSTRING, &params);
#else
  windowNewTextLabel(window, GPLSTRING, &params);
#endif

  params.gridY += 1;
  graphicsCheckbox =
    windowNewCheckbox(window, _("Boot in graphics mode"), &params);
  windowComponentSetSelected(graphicsCheckbox, bootGraphicsMode);
  windowRegisterEventHandler(graphicsCheckbox, &eventHandler);

  params.gridY += 1;
  params.gridWidth = 1;
  params.padBottom = 5;
  params.orientationX = orient_right;
  params.flags |= WINDOW_COMPFLAG_FIXEDWIDTH;
  params.font = NULL;
  okButton = windowNewButton(window, _("OK"), NULL, &params);
  windowRegisterEventHandler(okButton, &eventHandler);

  params.gridX = 1;
  params.orientationX = orient_left;
  cancelButton = windowNewButton(window, _("Cancel"), NULL, &params);
  windowRegisterEventHandler(cancelButton, &eventHandler);

  // No minimize or close buttons on the window
  windowRemoveMinimizeButton(window);
  windowRemoveCloseButton(window);
  windowComponentFocus(okButton);

  // Go
  windowSetVisible(window, 1);
}


int main(void)
{
  char *language = "";
  int diskManagerPid = 0;
  int logoWidth = 0;
  int logoPad = 0;
  int logoPos = 0;
  char title[80];
  char tmpChar[80];

  graphics = graphicsAreEnabled();
  bootGraphicsMode = graphics;

#ifdef BUILDLANG
  language=BUILDLANG;
#endif
  setlocale(LC_ALL, language);
  textdomain("partlogic");

  userLogin("admin", "");

#ifdef PLUS
  sprintf(title, "%s %s %s", WELCOMESTRING, PLUSSTRING, RELEASE);
#else
  sprintf(title, "%s %s", WELCOMESTRING, RELEASE);
#endif
  logoWidth = (strlen(title) + 10);

  if (graphics)
    {
      constructWindow(title);
      windowGuiRun();
      windowLogin("admin");
      windowDestroy(window);
    }

  else
    {
    restart:
      textScreenClear();

      // Use some extended ascii to make it prettier in text mode.  This
      // is convoluted -- not much use trying to understand it.
      logoPad = ((((logoWidth - strlen(title)) / 2)) - 1);
      logoPos = ((80 - logoWidth) / 2);
      memset(tmpChar, ' ', logoPos);
      tmpChar[logoPos++] = 201;
      memset((tmpChar + logoPos), 205, (logoWidth - 2));
      logoPos += (logoWidth - 2);
      tmpChar[logoPos++] = 187;
      tmpChar[logoPos++] = '\0';
      printf("\n%s\n", tmpChar);
      logoPos = ((80 - logoWidth) / 2);
      memset(tmpChar, ' ', logoPos);
      tmpChar[logoPos++] = 186;
      memset((tmpChar + logoPos), ' ', logoPad);
      logoPos += logoPad;
      strncpy((tmpChar + logoPos), title, strlen(title));
      logoPos += strlen(title);
      memset((tmpChar + logoPos), ' ', logoPad);
      logoPos += logoPad;
      tmpChar[logoPos++] = 186;
      tmpChar[logoPos++] = '\0';
      printf("%s\n", tmpChar);
      logoPos = ((80 - logoWidth) / 2);
      memset(tmpChar, ' ', logoPos);
      tmpChar[logoPos++] = 200;
      memset((tmpChar + logoPos), 205, (logoWidth - 2));
      logoPos += (logoWidth - 2);
      tmpChar[logoPos++] = 188;
      tmpChar[logoPos++] = '\0';
      printf("%s\n\n", tmpChar);

#ifdef PLUS
      printf("%s\n\n", GPLSHORTSTRING);
#else
      printf("%s\n\n", GPLSTRING);
#endif
      printf(_("Boot mode: %s [ Press 'G' to change at next boot ]\n\n"
	       "Continue? (y/n): "),
	     (bootGraphicsMode? _("GRAPHICS") : _("TEXT    ")));

      while (1)
	{
	  textInputSetEcho(0);
	  char c = getchar();
	  textInputSetEcho(1);

	  switch (c)
	    {
	    case 'y':
	    case 'Y':
	      printf("%s", _("Yes\n"));
	      break;
	    case 'n':
	    case 'N':
	      printf("%s", _("No\n"));
	      shutdown(1, 0);
	    case 'g':
	    case 'G':
	      if (bootGraphicsMode)
		bootGraphicsMode = 0;
	      else
		bootGraphicsMode = 1;
	      setGraphics(bootGraphicsMode);
	      goto restart;
	    }
	  break;
	}

#ifdef PLUS
      while (licenseCheckKeyValid() < 0)
	printf("\n\n%s: %s\n", _("Error"),
	       _("The license key you entered is not valid"));
#endif
    }

  while(1)
    {
      // Loop here, so that if the user closes the disk manager and then
      // cancels the shutdown, we will start a new disk manager.

      diskManagerPid = loaderLoadProgram("/programs/fdisk", 0);

      // Set the PID to the window manager thread
      userSetPid("admin", diskManagerPid);

      loaderExecProgram(diskManagerPid, 1);

      // If we return to here, the user has closed the disk manager.
      if (graphics)
	loaderLoadAndExec("/programs/shutdown", 0, 1);
      else
	shutdown(1, 0);
    }
}
